'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.CreateModel = undefined;

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Filters = require('../tools/Filters');

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Construct a method representation of a model. This method contains
 * no field information but rather config that is used when generating
 * a query.
 *
 * The resulting ModelFunction is an eternity method, meaning the result
 * of calling it is a new ModelFunction that contains the changes to the
 * original.
 */
var CreateModel = exports.CreateModel = function CreateModel(newModel) {
    var

    /* The ModelFunction that is returned. This stores information for
     * query generation such as the FieldName, ModelName of the data it
     * represents, query parameters and all children models it should
     * generate.
     * */Model = function Model(fieldName, modelParams) {
        for (var _len = arguments.length, childModels = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
            childModels[_key - 2] = arguments[_key];
        }

        var _normalizeFunctionPar = (0, _Filters.normalizeFunctionParameters)(fieldName, modelParams, childModels),
            name = _normalizeFunctionPar.name,
            params = _normalizeFunctionPar.params,
            models = _normalizeFunctionPar.models;

        var NextModel = _extends({}, Model);

        if (name) NextModel.FieldName = name;
        NextModel.Params = _extends({}, NextModel.Params, params);

        /* A utility method that recursively merges new child models
         * into the existing collection of child models.
         *
         * It uses the models' FieldName and not their ModelName
         * to match new => old as we do not want to overwrite
         * unrelated fields.
         * */
        var mergeChildren = function mergeChildren(oldChildren, newChildren) {
            return [].concat(_toConsumableArray((0, _filter3.default)(oldChildren, function (oldChildModel) {
                return !(0, _find3.default)(newChildren, function (newChildModel) {
                    return newChildModel.FieldName == oldChildModel.FieldName;
                });
            })), _toConsumableArray((0, _map3.default)(newChildren, function (newChildModel) {
                var previousChildModel = (0, _find3.default)(oldChildren, function (childModel) {
                    return childModel.FieldName == newChildModel.FieldName;
                });

                return previousChildModel ? CreateModel(_extends({}, previousChildModel, newChildModel, {
                    Children: mergeChildren(previousChildModel.Children, newChildModel.Children)
                })) : newChildModel;
            })));
        };

        NextModel.Children = mergeChildren(NextModel.Children, models);
        return CreateModel(NextModel);
    };

    if ((typeof newModel === 'undefined' ? 'undefined' : _typeof(newModel)) === 'object') {
        (0, _forEach3.default)(newModel, function (value, key) {
            return Model[key] = value;
        });
    } else {

        /* We want to store the model name and field name
         * separately as the field name can change on a per
         * query basis, while the model name must always
         * remain consistent.
         * */
        Model.ModelName = newModel;
        Model.FieldName = newModel;

        Model.Params = {};
        Model.Children = [];
        Model.Filters = {};
        Model.FieldParams = {};
    }

    var setFilter = function setFilter(key, value) {
        return CreateModel(_extends({}, Model, {
            Filters: _extends({}, Model.Filters, _defineProperty({}, key, value))
        }));
    };

    /* Model functions that allows for white-listing and
     * black-listing model fields on a per-query basis.
     * */
    Model.only = function (fields) {
        return setFilter("only", fields);
    };
    Model.without = function (fields) {
        return setFilter("without", fields);
    };
    Model.empty = function () {
        return setFilter("empty", true);
    };

    Model.fields = function (fieldParams) {
        return CreateModel(_extends({}, Model, {
            FieldParams: _extends({}, Model.FieldParams, fieldParams)
        }));
    };

    return Model;
};

exports.default = CreateModel;