'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _pickBy2 = require('lodash/pickBy');

var _pickBy3 = _interopRequireDefault(_pickBy2);

var _mapKeys2 = require('lodash/mapKeys');

var _mapKeys3 = _interopRequireDefault(_mapKeys2);

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _normalizr = require('normalizr');

var _ModelBuilder = require('./ModelBuilder');

var _ModelBuilder2 = _interopRequireDefault(_ModelBuilder);

var _RequestBuilder = require('./RequestBuilder');

var _RequestBuilder2 = _interopRequireDefault(_RequestBuilder);

var _Filters = require('../tools/Filters');

var _Fetch = require('../tools/Fetch');

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Modelizr = function () {
    function Modelizr(InitialClientState) {
        var _this = this;

        _classCallCheck(this, Modelizr);

        this.models = {};

        this.query = function () {
            return (0, _RequestBuilder2.default)(_this.ClientState, "query").apply(undefined, arguments);
        };

        this.mutate = function () {
            return (0, _RequestBuilder2.default)(_this.ClientState, "mutation").apply(undefined, arguments);
        };

        this.fetch = function () {
            return (0, _RequestBuilder2.default)(_this.ClientState, "fetch").apply(undefined, arguments);
        };

        if (!InitialClientState) throw new Error("Modelizr expects a Client State as its first parameter");

        this.ClientState = _extends({}, InitialClientState);

        /* order is important, all model schemas must exist before union schemas are created */
        this.generateModelFunctions();
        this.buildUnionSchemas();
        this.defineModelRelationships();

        var defaultConfig = {
            mock: false,
            debug: false,
            api: _Fetch.FETCH_API,
            throwOnErrors: true
        };

        this.ClientState.config = _extends({}, defaultConfig, InitialClientState.config || {});
        if (!this.ClientState.config.endpoint) throw new Error("Please provide a base endpoint to make queries to");

        this.normalizr = function (model) {
            return function (Data) {
                return Normalizr({
                    Data: Data,
                    ModelFunctions: [model],
                    ClientState: _this.ClientState
                });
            };
        };
    }

    /* Create ModelFunctions from the given model Data.
     *
     * If the DataType is a model (and not a union) then build
     * its normalizr schema. We do not create schemas' for unions until
     * after _all_ model schemas are present.
     * */


    _createClass(Modelizr, [{
        key: 'generateModelFunctions',
        value: function generateModelFunctions() {
            var _this2 = this;

            var models = this.ClientState.models;


            (0, _forEach3.default)(models, function (data, name) {
                var ModelData = _extends({}, data);

                if (!ModelData.normalizeAs) ModelData.normalizeAs = name;
                if (!ModelData.primaryKey) ModelData.primaryKey = "id";

                if (!ModelData._unionDataType) ModelData.normalizrSchema = new _normalizr.Schema(ModelData.normalizeAs, _extends({
                    idAttribute: ModelData.primaryKey
                }, ModelData.normalizrOptions || {}));

                _this2.ClientState.models[name] = ModelData;
                _this2.models[name] = (0, _ModelBuilder2.default)(name);
            });
        }

        /* Build all normalizr schemas for union DataTypes. A check
         * to make sure the union is nor referencing imaginary models
         * is performed.
         * */

    }, {
        key: 'buildUnionSchemas',
        value: function buildUnionSchemas() {
            var models = this.ClientState.models;


            (0, _forEach3.default)(models, function (ModelData, modelName) {
                if (ModelData._unionDataType) {
                    /* filter out all non-existing models and warn about them */
                    var ExistingModels = (0, _filter3.default)(ModelData.models, function (model) {
                        if (models[model]) return true;
                        // eslint-disable-next-line no-console
                        console.warn('Model "' + model + '" on union ' + modelName + ' points to an unknown model');
                    });

                    /* create a normalizr union */
                    ModelData.normalizrSchema = (0, _normalizr.unionOf)((0, _mapValues3.default)((0, _mapKeys3.default)(ExistingModels, function (model) {
                        return model;
                    }), function (model) {
                        return models[model].normalizrSchema;
                    }), { schemaAttribute: ModelData.schemaAttribute });
                }
            });
        }

        /* Recursively populate relationship information of each models
         * normalizr schema
         * */

    }, {
        key: 'defineModelRelationships',
        value: function defineModelRelationships() {
            var models = this.ClientState.models;


            /* Utility that flattens a field wrapped in an array */
            var unWrapArray = function unWrapArray(field) {
                return Array.isArray(field) ? { isArray: true, field: field[0] } : { isArray: false, field: field };
            };

            (0, _forEach3.default)(models, function (ModelData, modelName) {
                if (!ModelData._unionDataType) {
                    /* Filter out any model references that do not exist in our data set */
                    var ModelFields = (0, _pickBy3.default)(ModelData.fields, function (wrappedField, fieldName) {
                        var _unWrapArray = unWrapArray(wrappedField),
                            isArray = _unWrapArray.isArray,
                            field = _unWrapArray.field;

                        if (typeof field === 'string' && !(0, _Filters.isValidType)(field)) {
                            if (models[field]) return true;
                            // eslint-disable-next-line no-console
                            console.warn('Field { ' + fieldName + ': ' + (isArray ? "[" : "") + '"' + field + '"' + (isArray ? "]" : "") + ' } on \'' + modelName + '\' points to an unknown model');
                        }
                    });

                    /* Recursively define all model relationships on
                     * Model normalizr schemas
                     * */
                    ModelData.normalizrSchema.define((0, _mapValues3.default)(ModelFields, function (wrappedField) {
                        var _unWrapArray2 = unWrapArray(wrappedField),
                            isArray = _unWrapArray2.isArray,
                            field = _unWrapArray2.field;

                        var ChildModel = models[field];

                        if (isArray) return (0, _normalizr.arrayOf)(ChildModel.normalizrSchema);
                        return ChildModel.normalizrSchema;
                    }));
                }
            });
        }
    }]);

    return Modelizr;
}();

exports.default = Modelizr;