'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _pickBy2 = require('lodash/pickBy');

var _pickBy3 = _interopRequireDefault(_pickBy2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _times2 = require('lodash/times');

var _times3 = _interopRequireDefault(_times2);

var _join2 = require('lodash/join');

var _join3 = _interopRequireDefault(_join2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Filters = require('../tools/Filters');

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Utility method that generates a certain amount of spaces
 * in a string.
 */
var SPACES = 2;
var createIndent = function createIndent(spaces) {
    return (0, _join3.default)((0, _times3.default)(spaces * SPACES + 1, function () {
        return "";
    }), " ");
};

/* Construct a valid GraphQL parameter string from an object */
var buildParameters = function buildParameters(params) {
    if (!params || (0, _isEmpty3.default)(params)) return "";
    return '(' + (0, _map3.default)((0, _pickBy3.default)(params, function (param) {
        return param || param === false;
    }), function (param, key) {
        return key + ': ' + JSON.stringify(param).replace(/"([^(")"]+)":/g, "$1:");
    }) + ')';
};

/**
 * Generate a GraphQL query from a collection of modelizr models.
 */

exports.default = function (_ref) {
    var ClientState = _ref.ClientState,
        queryModels = _ref.queryModels,
        queryType = _ref.queryType,
        queryName = _ref.queryName,
        queryParams = _ref.queryParams;
    var models = ClientState.models;

    /* This compiles a FieldMap from a a collection of models. It is much
     * easier to generate a query from a normalized description of fields.
     * */

    var makeMap = function makeMap(queryModels) {
        var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
        return (0, _map3.default)(queryModels, function (ModelFunction) {
            var ModelData = models[ModelFunction.ModelName];

            /* Utility that strips modifier rejected fields. */
            var filter = function filter(fields) {
                return (0, _pickBy3.default)(fields, function (type, field) {
                    var _ModelFunction$Filter = ModelFunction.Filters,
                        only = _ModelFunction$Filter.only,
                        without = _ModelFunction$Filter.without,
                        empty = _ModelFunction$Filter.empty;

                    if (only) return (0, _find3.default)(only, function (_field) {
                        return _field == field;
                    });
                    if (without) return !(0, _find3.default)(without, function (_field) {
                        return _field == field;
                    });
                    if (empty) return false;
                    return true;
                });
            };

            /* Filter out fields that have been rejected via modifiers and
             * strip any model relationships from the fields and recursively
             * generate a FieldMap.
             * */
            var pruneFields = function pruneFields(fields) {
                return (0, _map3.default)(filter((0, _Filters.getPlainFields)(fields)), function (type, field) {
                    /* This variable is used to keep track of collection object-type fields.
                     * These fields should not have a sub-selection of fields in the GraphQL
                     * query, but should still be defined in our model for accurate mock
                     * generation.
                     * */
                    var arrayType = false;

                    if (Array.isArray(type)) {
                        type = type[0];
                        arrayType = true;
                    }

                    /* Utility that allows adding parameters to individual fields. */
                    var addParams = function addParams(field) {
                        var fieldParams = ModelFunction.FieldParams;
                        var findParams = function findParams(field) {
                            return (0, _find3.default)(fieldParams, function (params, fieldName) {
                                return fieldName == field;
                            });
                        };

                        if (typeof field === 'string') {
                            var _params = findParams(field);
                            if (_params) return { fields: [], name: field, params: _params };
                        } else {
                            var _params2 = findParams(field.name);
                            if (_params2) return _extends({}, field, { params: _params2 });
                        }

                        return field;
                    };

                    /* We check for a __alias property on the field type.
                     * If one if found, use it instead of the fieldName
                     * */
                    if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) === 'object' && type.__alias) field = field + ': ' + type.__alias;
                    return addParams((0, _Filters.isValidType)(type) || arrayType ? field : { name: field, fields: pruneFields(type) });
                });
            };

            return {
                name: '' + (prefix ? "... on " : "") + ModelFunction.FieldName,
                params: ModelFunction.Params,
                fields: [].concat(_toConsumableArray(pruneFields(ModelData.fields)), _toConsumableArray(makeMap(ModelFunction.Children, ModelData._unionDataType)))
            };
        });
    };

    var FieldMaps = makeMap(queryModels);

    /* Generate an indented and multi-lined GraphQL query string
     * from our FieldMap. The type and name of the generated
     * query will be determined based on the queryType and queryName
     * parameters.
     * */
    var GenerateFields = function GenerateFields(FieldMap) {
        var indent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 2;
        var name = FieldMap.name,
            fields = FieldMap.fields,
            params = FieldMap.params;

        var length = !!fields.length;

        return '\n' + createIndent(indent - 1) + name + buildParameters(params) + ' ' + (length ? "{" : "") + (0, _map3.default)(fields, function (field) {
            return typeof field === 'string' ? '\n' + createIndent(indent) + field : '' + GenerateFields(field, indent + 1);
        }) + '\n' + createIndent(indent - 1) + (length ? "}" : "");
    };

    return queryType + ' ' + (queryName || 'modelizr_' + queryType) + buildParameters(queryParams) + ' {' + (0, _map3.default)(FieldMaps, function (FieldMap) {
        return GenerateFields(FieldMap);
    }) + '\n}';
};