'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _curry2 = require('lodash/curry');

var _curry3 = _interopRequireDefault(_curry2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Filters = require('../tools/Filters');

var _logger = require('../tools/logger');

var _QueryGeneration = require('../core/QueryGeneration');

var _QueryGeneration2 = _interopRequireDefault(_QueryGeneration);

var _Normalization = require('../data/Normalization');

var _Normalization2 = _interopRequireDefault(_Normalization);

var _Mocks = require('../data/Mocks');

var _Mocks2 = _interopRequireDefault(_Mocks);

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (ClientState, queryType) {
    return function (queryName, queryParams) {
        for (var _len = arguments.length, queryModels = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
            queryModels[_key - 2] = arguments[_key];
        }

        var _normalizeFunctionPar = (0, _Filters.normalizeFunctionParameters)(queryName, queryParams, queryModels),
            name = _normalizeFunctionPar.name,
            params = _normalizeFunctionPar.params,
            models = _normalizeFunctionPar.models;

        /* If debugging is enabled, create a logger instance, else
         * create a noop debugger
         * */


        var logger = ClientState.config.debug ? (0, _logger.createLogger)('[' + queryType + ': ' + (0, _map3.default)(models, function (model) {
            return model.FieldName;
        })) : {
            add: function add() {},
            print: function print() {}
        };

        /* generate the query and add it to the request body. If the
         * requestType is 'fetch', do not mutate the body
         * */
        var query = (0, _QueryGeneration2.default)({ ClientState: ClientState, queryModels: models, queryType: queryType, queryName: name, queryParams: params });
        var config = _extends({}, ClientState.config, {
            body: queryType == 'fetch' ? {} : { query: query }
        });

        /* Add the generated query to the debugging instance */
        if (queryType != 'fetch') logger.add("Query", query);

        /* A utility method that, when given a response - calls our internal
         * Normalize method with both the given response and the model tree.
         * */
        var _normalize = function _normalize(Data) {
            return (0, _Normalization2.default)({
                Data: Data,
                ModelFunctions: models,
                ClientState: ClientState
            });
        };

        // eslint-disable-next-line prefer-const
        var REQUEST = void 0;

        /* A utility method that mutates config properties and returns the REQUEST object.
         * This is to allow for chaining config modifiers (.api().headers() ...)
         * */
        var setConfig = (0, _curry3.default)(function (key, useDefault, value) {
            if (!value && value !== false) {
                if (useDefault) {
                    value = useDefault;
                } else {
                    throw new Error('Please provide a value for the modifier \'' + key + '()\'');
                }
            }
            config[key] = value;
            return REQUEST;
        });

        /* A utility method that calls the configured api and
         * adds the response to the debugger instance.
         * */
        var MAKE_REQUEST = function MAKE_REQUEST() {
            return (config.mock ? (0, _Mocks2.default)(_extends({}, ClientState, { config: config }), models) : config.api(config)).then(function (res) {
                logger.add("Server Response", res.server_response || {});
                logger.add("GraphQL response", {
                    data: res.data,
                    errors: res.errors
                });

                return res;
            }).catch(function (e) {
                logger.add("error", e);
                logger.print();
                throw e;
            });
        };

        /* Our request object that the user has access to. This object
         * contains all modifier methods as well the request catalysts,
         * .normalize() and .then()
         * */
        REQUEST = {
            api: setConfig("api", false),
            endpoint: setConfig("endpoint", false),
            headers: function headers(_headers) {
                return setConfig("headers", false)(_extends({}, config.headers, _headers));
            },
            method: setConfig("method", "POST"),
            mock: setConfig("mock", true),
            debug: setConfig("debug", true),
            body: setConfig("body", {}),
            throwOnErrors: setConfig("throwOnErrors", true),

            /* give the generated query to the provided callback and return the REQUEST object */
            generate: function generate(cb) {
                if (typeof cb !== 'function') throw new Error("A function needs to be provided when calling .generate");
                cb(query);
                return REQUEST;
            },

            then: function then(cb) {
                return MAKE_REQUEST().then(function (res) {
                    logger.print();
                    return cb(res, _normalize);
                });
            },
            normalize: function normalize(cb) {
                return MAKE_REQUEST().then(function (res) {
                    var normalizedResponse = _normalize(res.data);
                    logger.add("Normalized Response", normalizedResponse);
                    logger.print();
                    return cb(_extends({}, res, normalizedResponse));
                });
            }
        };

        return REQUEST;
    };
};