'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _mapKeys2 = require('lodash/mapKeys');

var _mapKeys3 = _interopRequireDefault(_mapKeys2);

var _times2 = require('lodash/times');

var _times3 = _interopRequireDefault(_times2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _sample2 = require('lodash/sample');

var _sample3 = _interopRequireDefault(_sample2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _Filters = require('../tools/Filters');

var _DataGeneration = require('./DataGeneration');

var _uuid = require('../tools/uuid');

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Given a Data Description and a collection of query models, recursively
 * generate data that matches the type-specified of the model.
 *
 * This function will return a promise as it is mocking a fetch request.
 */
exports.default = function (clientState, queryModels) {
    var mockConfig = clientState.config.mock;

    var generate = (0, _DataGeneration.generator)(clientState.config.faker);

    var MockModels = function MockModels(models) {
        var mock = function mock(Model) {
            var CurrentModel = Model;
            var fieldsToMock = {};
            var ModelData = false;
            var schemaAttribute = null;

            /* Check to see if we are mocking a Model or a nested set of fields */
            if (typeof Model === 'function') {
                ModelData = clientState.models[Model.ModelName];

                /* If the model being mocked is a Union, then it has no actual 'fields'
                 * that we can mock. We need to keep track of its schemaAttribute and
                 * then randomly select one of its Child Models. We override the
                 * currently set ModelFunction with this chosen Model.
                 * */
                if (ModelData._unionDataType) {
                    if (!Model.Children.length) throw new Error('No children were given to union ' + Model.FieldName);
                    schemaAttribute = ModelData.schemaAttribute;
                    CurrentModel = (0, _sample3.default)(Model.Children);
                    ModelData = clientState.models[CurrentModel.ModelName];

                    /* If the schemaAttribute of the union is a function, then look for
                     * a schemaType on the chosen model.
                     * */
                    if (typeof schemaAttribute === 'function') schemaAttribute = ModelData.schemaType;
                }
                fieldsToMock = (0, _Filters.getPlainFields)(ModelData.fields);
            } else {
                fieldsToMock = (0, _Filters.getPlainFields)(Model);
            }

            /* Map over the filtered set of fields and generate information
             * based on their data type. If the field is a nested set of fields,
             * pass that field back into our mock function.
             * */
            var mockedFields = (0, _mapValues3.default)(fieldsToMock, function (field) {
                var generateOrMock = function generateOrMock(type) {
                    if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) === 'object' && !(0, _Filters.isValidType)(type)) return mock(type);
                    return generate(type);
                };

                if (Array.isArray(field)) {
                    field = field[0];
                    return (0, _map3.default)((0, _times3.default)(10), function () {
                        return generateOrMock(field);
                    });
                }

                return generateOrMock(field);
            });

            /* If this model is querying child models, then they also need
             * to be mocked. We first check their relationship description
             * to figure out if they should be mocked as a collection or as
             * a single entity.
             * */
            var KeyedFunctions = (0, _mapKeys3.default)(CurrentModel.Children, function (model) {
                return model.FieldName;
            });
            var mockedChildren = (0, _mapValues3.default)(KeyedFunctions, function (model, fieldName) {
                if (ModelData && ModelData.fields && Array.isArray(ModelData.fields[fieldName])) return (0, _map3.default)((0, _times3.default)(10), function () {
                    return mock(model);
                });
                return mock(model);
            });

            mockedFields = _extends({}, mockedFields, mockedChildren);

            /* Replace the generated primaryKey data with a V4 UUID string and, if
             * the model is a union type, set its schemaAttribute accordingly.
             * */
            if (ModelData && mockedFields[ModelData.primaryKey]) mockedFields[ModelData.primaryKey] = (0, _uuid.v4)();
            if (schemaAttribute) mockedFields[schemaAttribute] = CurrentModel.ModelName;

            return mockedFields;
        };

        /* We look at mock config to determine if we should generate an array
         * of data or single entities
         * */
        var KeyedFunctions = (0, _mapKeys3.default)(models, function (model) {
            return model.FieldName;
        });
        return (0, _mapValues3.default)(KeyedFunctions, function (model, field) {
            if ((typeof mockConfig === 'undefined' ? 'undefined' : _typeof(mockConfig)) === 'object') {
                if (mockConfig[field] && mockConfig[field] === Array) return (0, _map3.default)((0, _times3.default)(10), function () {
                    return mock(model);
                });
            }
            return mock(model);
        });
    };

    return new Promise(function (resolve) {
        return resolve({
            server_response: {},
            data: MockModels(queryModels),
            errors: null
        });
    });
};