'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _mapKeys2 = require('lodash/mapKeys');

var _mapKeys3 = _interopRequireDefault(_mapKeys2);

var _normalizr = require('normalizr');

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A function that normalizes data based on specified model relationships using
 * normalizr.
 *
 * This will map over the provided models and pass in a normalizr schema containing
 * each models schema. If the entities from the response that match the model are
 * an array, the schema will be wrapped in arrayOf().
 */
exports.default = function (_ref) {
    var Data = _ref.Data,
        ModelFunctions = _ref.ModelFunctions,
        ClientState = _ref.ClientState;
    var models = ClientState.models;

    /* Create a map of ModelFunctions who's keys match
     * those of the Data result
     * */

    var KeyedFunctions = (0, _mapKeys3.default)(ModelFunctions, function (ModelFunction) {
        return ModelFunction.FieldName;
    });

    if (!Data) return {};
    return (0, _normalizr.normalize)(Data, (0, _mapValues3.default)(KeyedFunctions, function (ModelFunction) {
        var Entities = Data[ModelFunction.FieldName];
        var ModelData = models[ModelFunction.ModelName];

        if (Array.isArray(Entities)) return (0, _normalizr.arrayOf)(ModelData.normalizrSchema);
        return ModelData.normalizrSchema;
    }));
};