'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.normalizeFunctionParameters = exports.getPlainFields = exports.isValidType = undefined;

var _pickBy2 = require('lodash/pickBy');

var _pickBy3 = _interopRequireDefault(_pickBy2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _types = require('../types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Checks if a piece of information is a valid data description
 */
var isValidType = exports.isValidType = function isValidType(type) {
    if (Array.isArray(type)) {
        type = type[0];
    }

    if ((typeof type === 'undefined' ? 'undefined' : _typeof(type)) === 'object') {
        if (type.__type) type = type.__type;
    }

    var TypeMap = [Boolean, "boolean", String, "string", Number, "number", "integer", "float"];

    return (0, _find3.default)(TypeMap, function (_type) {
        return _type === type;
    });
};

/**
 * Strips a collection of fields of all model relationships
 */
var getPlainFields = exports.getPlainFields = function getPlainFields(fields) {
    return (0, _pickBy3.default)(fields, function (field) {
        var check = function check(field) {
            return (typeof field === 'undefined' ? 'undefined' : _typeof(field)) === "object" || isValidType(field);
        };
        if (Array.isArray(field)) return check(field[0]);
        return check(field);
    });
};

/**
 * Given three parameters, figure out the type of each param and return
 * a corrected set of parameters.
 */
var normalizeFunctionParameters = exports.normalizeFunctionParameters = function normalizeFunctionParameters(name, params, models) {
    var trueModels = models;
    var trueName = void 0,
        trueParams = {};

    if (typeof name === 'string') {
        trueName = name;

        if (typeof params === 'function') trueModels.unshift(params);
        if ((typeof params === 'undefined' ? 'undefined' : _typeof(params)) === 'object') trueParams = params;
    } else {
        if (params) trueModels.unshift(params);

        if (typeof name === 'function') trueModels.unshift(name);
        if ((typeof name === 'undefined' ? 'undefined' : _typeof(name)) === 'object') trueParams = name;
    }

    return { name: trueName, params: trueParams, models: trueModels };
};